"""conftest for parse_completion_status_service tests."""

from datetime import datetime
from pathlib import Path

import pytest

from cg.utils.time import DATE_TIME_FORMAT


@pytest.fixture()
def run_completion_status_path(novaseq_x_flow_cell_dir) -> Path:
    """Return the path to the run completion status file."""
    return Path(
        novaseq_x_flow_cell_dir,
        "RunCompletionStatus.xml",
    )


@pytest.fixture()
def expected_start_time() -> datetime:
    """Return the expected start time."""
    return datetime.strptime("2023-11-08T16:06:11", DATE_TIME_FORMAT)


@pytest.fixture()
def expected_end_time() -> datetime:
    """Return the expected end time."""
    return datetime.strptime("2023-11-09T15:24:13", DATE_TIME_FORMAT)



"""This script tests the cli methods to get analysis in status-db"""

from click.testing import CliRunner

from cg.cli.get import get
from cg.constants import EXIT_SUCCESS
from cg.models.cg_config import CGConfig
from cg.store.models import Analysis
from cg.store.store import Store
from tests.store_helpers import StoreHelpers


def test_get_analysis_bad_case(cli_runner: CliRunner, base_context: CGConfig):
    """Test to get a analysis using a non-existing analysis-id"""
    # GIVEN an empty database

    # WHEN getting a analysis
    name = "dummy_name"
    result = cli_runner.invoke(get, ["analysis", name], obj=base_context)

    # THEN it should error about missing case instead of getting a analysis
    assert result.exit_code != EXIT_SUCCESS


def test_get_analysis_required(
    cli_runner: CliRunner, base_context: CGConfig, disk_store: Store, helpers: StoreHelpers
):
    """Test to get a analysis using only the required argument"""
    # GIVEN a database with an analysis
    analysis: Analysis = helpers.add_analysis(disk_store, workflow_version="9.3")
    internal_id = analysis.case.internal_id
    assert disk_store._get_query(table=Analysis).count() == 1

    # WHEN getting a analysis
    result = cli_runner.invoke(get, ["analysis", internal_id], obj=base_context)

    # THEN it should have been gotten
    assert result.exit_code == EXIT_SUCCESS
    assert str(analysis.started_at) in result.output
    assert analysis.workflow in result.output
    assert analysis.workflow_version in result.output



"""Module to test the parse_completion_status_service"""

from datetime import datetime
from pathlib import Path

from cg.services.parse_run_completion_status_service.parse_run_completion_status_service import (
    ParseRunCompletionStatusService,
)


def test_get_start_time(run_completion_status_path: Path, expected_start_time: datetime):
    """Test to get the start time."""
    # GIVEN a parse run completion status service and a path to the run completion status file
    parser = ParseRunCompletionStatusService()

    # WHEN getting the start time
    start_time: datetime = parser.get_start_time(
        run_completion_status_path=run_completion_status_path
    )

    # THEN assert the start time is correct
    assert start_time == expected_start_time


def test_get_end_time(run_completion_status_path: Path, expected_end_time: datetime):
    """Test to get the end time."""
    # GIVEN a parse run completion status service and a path to the run completion status file
    parser = ParseRunCompletionStatusService()

    # WHEN getting the end time
    end_time: datetime = parser.get_end_time(run_completion_status_path=run_completion_status_path)

    # THEN assert the end time is correct
    assert end_time == expected_end_time



"""Service to parse the RunInfo.xml file."""

from cg.constants.demultiplexing import RunCompletionStatusNodes
from cg.io.xml import read_xml
from cg.utils.time import format_time_from_string
from datetime import datetime
from pathlib import Path
from xml.etree.ElementTree import ElementTree


class ParseRunCompletionStatusService:

    @staticmethod
    def _parse_run_completion_status_path(run_completion_status_path: Path) -> ElementTree:
        """Parse the RunInfo.xml file."""
        return read_xml(file_path=run_completion_status_path)

    def get_start_time(self, run_completion_status_path: Path) -> datetime:
        """Get the sequencer start date and time."""
        tree: ElementTree = self._parse_run_completion_status_path(run_completion_status_path)
        return format_time_from_string(tree.find(RunCompletionStatusNodes.RUN_START).text)

    def get_end_time(self, run_completion_status_path: Path) -> datetime:
        """Get the sequencer end date and time."""
        tree: ElementTree = self._parse_run_completion_status_path(run_completion_status_path)
        return format_time_from_string(tree.find(RunCompletionStatusNodes.RUN_END).text)
